package com.company;


import com.rti.dds.domain.DomainParticipant;
import com.rti.dds.domain.DomainParticipantFactory;
import com.rti.dds.infrastructure.StatusKind;
import com.rti.dds.subscription.*;
import com.rti.dds.topic.Topic;
import com.rti.hello.idl.*;

public class Main {

    public static void main(String[] args) {

        System.out.println("Creating DDS entities");

        DomainParticipant dp1 = DomainParticipantFactory.get_instance().create_participant(
                44, DomainParticipantFactory.PARTICIPANT_QOS_DEFAULT,
                null, StatusKind.STATUS_MASK_NONE);
        if (dp1 == null)
            throw new RuntimeException();

        DomainParticipant dp2 = DomainParticipantFactory.get_instance().create_participant(
                44, DomainParticipantFactory.PARTICIPANT_QOS_DEFAULT,
                null, StatusKind.STATUS_MASK_NONE);
        if (dp2 == null)
            throw new RuntimeException();

        HelloWorldTypeSupport.register_type(dp1,
                HelloWorldTypeSupport.get_type_name());

        HelloWorldTypeSupport.register_type(dp2,
                HelloWorldTypeSupport.get_type_name());

        Topic topic1 = dp1.create_topic(
                "MyTopic",
                HelloWorldTypeSupport.get_type_name(),
                DomainParticipant.TOPIC_QOS_DEFAULT,
                null, // listener
                StatusKind.STATUS_MASK_NONE);

        Topic topic2 = dp2.create_topic(
                "MyTopic",
                HelloWorldTypeSupport.get_type_name(),
                DomainParticipant.TOPIC_QOS_DEFAULT,
                null, // listener
                StatusKind.STATUS_MASK_NONE);

        HelloWorldDataWriter dataWriter =
                (HelloWorldDataWriter) dp1.create_datawriter_with_profile(
                        topic1,
                        "mylib", "profile_HelloWorld",
                        null, // listener
                        StatusKind.STATUS_MASK_NONE);
        if (dataWriter == null)
            throw new RuntimeException();

        HelloWorldDataReader dataReader =
                (HelloWorldDataReader) dp2.create_datareader_with_profile(
                        topic2,
                        "mylib", "profile_HelloWorld",
                        null,         // Listener
                        StatusKind.STATUS_MASK_ALL);
        if (dataReader == null)
            throw new RuntimeException();

        System.out.println("Waiting for discovery");


        sleep(2000);

        System.out.println("publishing");

        int id = 0;
        long time = System.currentTimeMillis();

        while (true) {
            for (int j = 0; j < 2; j++) {
                HelloWorld helloWorld = new HelloWorld();
                helloWorld.sampleId = ++id;
                dataWriter.write(helloWorld, null);
            }
            sleep(30);
            if (System.currentTimeMillis() - time > 10000)
                break;
        }

        System.out.println("written: " + id);

        sleep(8000);

        HelloWorldSeq dataSeq = new HelloWorldSeq();
        SampleInfoSeq sampleInfoSeq = new SampleInfoSeq();
        dataReader.take(dataSeq, sampleInfoSeq, 1000,
                SampleStateKind.ANY_SAMPLE_STATE, ViewStateKind.ANY_VIEW_STATE,
                InstanceStateKind.ANY_INSTANCE_STATE);

        for (Object obj : dataSeq)
        {
            HelloWorld helloWorld = (HelloWorld) obj;
            System.out.print(helloWorld.sampleId + " ");
        }

        System.out.println();
        System.out.println("read: " + dataSeq.size());
    }

    private static void sleep(int millis) {
        try {
            Thread.sleep(millis);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }
}
